-- Import addon data
local _, addonData = ...;
-- Frames
local loginFrame = CreateFrame("Frame");
local questFrame = CreateFrame("Frame");
-- Events
local events = {};

-- Tables
local completedStorylines = {};

local function isActiveQuestCompleted(index)
	local name, level, isTrivial, isComplete, isLegendary, isIgnored = select(((index * 6) - 6) + 1, GetGossipActiveQuests());
	return isComplete;
end

function events:GOSSIP_SHOW()
	--print("GOSSIP!");
	
	local activeQuests = GetNumGossipActiveQuests();
	local availableQuests = GetNumGossipAvailableQuests();
	
	if (activeQuests > 0) then
		-- Loop through all available quests
		for i = 1, activeQuests, 1 do
			local isCompleted = isActiveQuestCompleted(i);
			
			if (isCompleted) then
				SelectGossipActiveQuest(i);
			end
		end
	elseif (availableQuests > 0) then
		-- Loop through all active quests
		for i = 1, availableQuests, 1 do
			SelectGossipAvailableQuest(i);
		end
	end
end

function events:QUEST_GREETING()
	local activeQuests = GetNumActiveQuests();
	local availableQuests = GetNumAvailableQuests();
	
	--print(format("GREETING fired, available quests: %i | active quests: %i", availableQuests, activeQuests));
	
	for i = 1, activeQuests do
		SelectActiveQuest(i);
	end
	
	for i = 1, availableQuests do
		SelectAvailableQuest(i);
	end
end

function events:QUEST_DETAIL()
	--print("DETAILS!");
	AcceptQuest()
end

function events:QUEST_PROGRESS()
	--print("PROGRESS!");
	local isCompletable = IsQuestCompletable()
	
	if (isCompletable) then
		CompleteQuest()
	end
end

function events:QUEST_COMPLETE()
	local choices = GetNumQuestChoices();
	
	--print(format("Quest completion event raised! Amount of rewards: %i", choices));
	
	local choice = 1;
	if (choices < 2) then
		GetQuestReward(choice)
		--print("Quest completed!");
	end
	
	
	--[[
	if (choices > 1) then
		-- Find most suitable choice
	else
		GetQuestReward(choice)
	end
	]]
end

function events:QUESTLINE_UPDATE()
	local mapId = C_Map.GetBestMapForUnit("player");
	local storyAchievementID, storyMapID = C_QuestLog.GetZoneStoryInfo(mapId);
	if (storyAchievementID) then
		local mapInfo = C_Map.GetMapInfo(storyMapID);
		local numCriteria = GetAchievementNumCriteria(storyAchievementID);
		
		local completedCriteria = 0;
		for i = 1, numCriteria do
			local _, _, completed = GetAchievementCriteriaInfo(storyAchievementID, i);
			if ( completed ) then
				completedCriteria = completedCriteria + 1;
			end
		end
		
		if (numCriteria == completedCriteria) then
			DLU.createNotification("You have completed the storyline in this area.");
		end
	end
end

-- Login
loginFrame:RegisterEvent("PLAYER_LOGIN");
loginFrame:SetScript("OnEvent", function()
	
	if (DLUCharacterSettings.questingMode.value) then
		questFrame:SetScript("OnEvent", function(self, event, ...)
			events[event](self, ...);
		end);
		for k, v in pairs(events) do
			questFrame:RegisterEvent(k);
		end
	end
	
	loginFrame:UnregisterEvent("PLAYER_LOGIN")
end);